unit parallelTask1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls,
  MMSHelpers,
  OtlCommon,
  OtlComm,
  OtlCollections,
  OtlTask,
  OtlTaskControl,
  OtlParallel;

const
  WM_RETRIEVED = WM_USER;
  WM_STOPPED = WM_USER + 1;

type
  TfrmLinkRetrieval = class(TForm)
    btnStart: TButton;
    lbLinks: TListBox;
    procedure btnStartClick(Sender: TObject);
  private
    FParaTask: IOmniParallelTask;
    FRequests: IOmniBlockingCollection;
    function CreateRequests: IOmniBlockingCollection;
    procedure MarkRetrieved(const task: IOmniTaskControl; const msg: TOmniMessage);
    procedure RetrievePages(const task: IOmniTask);
    procedure TaskStopped(var msg: TMessage); message WM_STOPPED;
  end;

var
  frmLinkRetrieval: TfrmLinkRetrieval;

implementation

uses
  StrUtils;

{$R *.dfm}

const
  CLinks: array [1..16] of string = ('97248', '97242', '97229', '97224', '97211',
    '97214', '97193', '97183', '97173', '97174', '97169', '97156', '97150', '97149',
    '97147', '97144');

function MakeUrl(const page: string): string;
begin
  Result := 'http://www.delphifeeds.com/go/s/' + page;
end;

{ TfrmLinkRetrieval }

procedure TfrmLinkRetrieval.btnStartClick(Sender: TObject);
var
  taskConfig: IOmniTaskConfig;
begin
  FRequests := CreateRequests;

  taskConfig := Parallel.TaskConfig;
  taskConfig.OnMessage(WM_RETRIEVED, MarkRetrieved);

  FParaTask := Parallel.ParallelTask;
  FParaTask.NoWait;
  FParaTask.TaskConfig(taskConfig);
  FParaTask.OnStop(
    procedure
    begin
      PostMessage(frmLinkRetrieval.Handle, WM_STOPPED, 0, 0);
    end);
  FParaTask.NumTasks(4).Execute(RetrievePages);
end;

function TfrmLinkRetrieval.CreateRequests: IOmniBlockingCollection;
var
  i: integer;
begin
  Result := TOmniBlockingCollection.Create;

  lbLinks.Clear;
  for i := Low(CLinks) to High(CLinks) do begin
    Result.Add(CLinks[i]);
    lbLinks.Items.Add('? ' + MakeUrl(CLinks[i]));
  end;

  Result.CompleteAdding; // mark the collection 'full'
end;

procedure TfrmLinkRetrieval.MarkRetrieved(const task: IOmniTaskControl; const msg: TOmniMessage);
var
  idxItem: integer;
  s      : string;
begin
  idxItem := lbLinks.Items.IndexOf('? ' + MakeUrl(msg.MsgData[0].AsString));

  s := lbLinks.Items[idxItem];
  s[1] := '+';
  lbLinks.Items[idxItem] := s;
end;

procedure TfrmLinkRetrieval.RetrievePages(const task: IOmniTask);
begin
  // implement
end;

procedure TfrmLinkRetrieval.TaskStopped(var msg: TMessage);
begin
  if assigned(FParaTask) then begin
    FParaTask.WaitFor(INFINITE);
    FParaTask := nil;
  end;
end;

end.
